/*  add.cc - this file is part of MediaTomb.
                                                                                
    Copyright (C) 2005 Gena Batyan <bgeradz@deadlock.dhs.org>,
                       Sergey Bostandzhyan <jin@deadlock.dhs.org>
                                                                                
    MediaTomb is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                
    MediaTomb is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
                                                                                
    You should have received a copy of the GNU General Public License
    along with MediaTomb; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "server.h"
#include <stdio.h>
#include "common.h"
#include "storage.h"
#include "cds_objects.h"
#include "dictionary.h"
#include "pages.h"
#include "content_manager.h"
#include "session_manager.h"

using namespace zmm;
using namespace mxml;

web::add::add() : WebRequestHandler()
{}

void web::add::process()
{
    Ref<Session>   session;
    Ref<Storage>   storage;
    session_data_t sd;

    check_request();
    
    String object_id = param("object_id");
    String path = param("path");
    String driver = param("driver");
    String sid = param("sid");

    if (driver == "1")
    {
        storage = Storage::getInstance();
        // depending on the driver we store the data in different
        // locations within the session
        sd = PRIMARY;
    }
    else if (driver == "2")
    {
        storage = Storage::getInstance(FILESYSTEM_STORAGE);
        sd = SECONDARY;
    }

    // there must at least a path or an object_id given
    if (((object_id == nil) || (object_id == "")) && ((path == nil) || (path == "")))
        throw Exception(String("missing parameters: either path or object id must be specified"));


    // if path is there, we take it, else browse...
    if ((path == nil) || (path == ""))
    {
        Ref<CdsObject> current = storage->loadObject(object_id);
        path = current->getLocation();
    }
    
    Ref<ContentManager> cm = ContentManager::getInstance();
    cm->addFile(path, true);

    session = SessionManager::getInstance()->getSession(sid);

    object_id = session->getFrom(sd, "object_id");
    if (object_id == nil) object_id = "0";
    
    
    Ref<Dictionary> sub(new Dictionary());
    sub->put("object_id", object_id);
    sub->put("driver", driver);
    sub->put("sid", sid); 
    *out << subrequest("browse", sub);
}

